@@ -15,23 +15,20 @@
  */
 
 import { defineMessages } from 'react-intl';
-import { fromJS } from 'immutable';
-import { normalize, arrayOf } from 'normalizr';
+import { normalize } from 'normalizr';
 import when from 'when';
 import yaml from 'js-yaml';
 
-import logger from '../services/logger';
+import { handleErrors } from './ErrorActions';
 import MistralApiService from '../services/MistralApiService';
-import MistralApiErrorHandler from '../services/MistralApiErrorHandler';
 import NotificationActions from '../actions/NotificationActions';
 import PlansConstants from '../constants/PlansConstants';
 import { planFileSchema } from '../normalizrSchemas/plans';
 import StackActions from '../actions/StacksActions';
-import SwiftApiErrorHandler from '../services/SwiftApiErrorHandler';
 import SwiftApiService from '../services/SwiftApiService';
 import MistralConstants from '../constants/MistralConstants';
-import { getAppConfig } from '../services/utils';
 import { PLAN_ENVIRONMENT } from '../constants/PlansConstants';
+import { getServiceUrl } from '../selectors/auth';
 
 const messages = defineMessages({
   planCreatedNotificationTitle: {
@@ -50,6 +47,10 @@ const messages = defineMessages({
     id: 'PlansActions.planUpdatedNotificationMessage',
     defaultMessage: 'The plan {planName} was successfully updated.'
   },
+  planUpdateFailed: {
+    id: 'PlansActions.planUpdateFailedNotificationMessage',
+    defaultMessage: 'Plan update failed'
+  },
   planDeletedNotificationTitle: {
     id: 'PlansActions.planDeletedNotificationTitle',
     defaultMessage: 'Plan Deleted'
@@ -85,42 +86,39 @@ export default {
   fetchPlans() {
     return dispatch => {
       dispatch(this.requestPlans());
-      MistralApiService.runAction(MistralConstants.PLAN_LIST)
-        .then(response => {
-          const planNames = JSON.parse(response.output).result;
-          when
-            .all(
-              planNames.map(name =>
-                SwiftApiService.getObject(name, PLAN_ENVIRONMENT)
+      return (
+        dispatch(MistralApiService.runAction(MistralConstants.PLAN_LIST))
+          // TODO(jtomasek): This block should be done on Mistral action side
+          .then(planNames =>
+            when
+              .all(
+                planNames.map(name =>
+                  dispatch(SwiftApiService.getObject(name, PLAN_ENVIRONMENT))
+                )
               )
-            )
-            .then(responses => {
-              const plans = responses.map(response => {
-                const { name, description } = yaml.safeLoad(
-                  response.responseText
+              .then(planEnvs =>
+                planNames.map(planName => {
+                  for (let i = 0; i < planEnvs.length; i++) {
+                    const { name, description } = yaml.safeLoad(planEnvs[i]);
+                    if (name === planName) {
+                      return { name, description };
+                    }
+                  }
+                  return { name: planName };
+                })
+              )
+              .catch(error => {
+                dispatch(
+                  handleErrors(error, 'Plan descriptions could not be loaded')
                 );
-                return { name, description };
-              });
-              dispatch(this.receivePlans(plans));
-            })
-            .catch(error => {
-              logger.error(
-                'Error in PlansActions.fetchPlans',
-                error.stack || error
-              );
-              dispatch(this.receivePlans(planNames.map(name => ({ name }))));
-            });
-        })
-        .catch(error => {
-          logger.error(
-            'Error in PlansActions.fetchPlans',
-            error.stack || error
-          );
-          let errorHandler = new MistralApiErrorHandler(error);
-          errorHandler.errors.forEach(error => {
-            dispatch(NotificationActions.notify(error));
-          });
-        });
+                return planNames.map(name => ({ name }));
+              })
+          )
+          .then(plans => dispatch(this.receivePlans(plans)))
+          .catch(error =>
+            dispatch(handleErrors(error, 'Plans could not be loaded'))
+          )
+      );
     };
   },
 
@@ -143,24 +141,14 @@ export default {
   fetchPlan(planName) {
     return dispatch => {
       dispatch(this.requestPlan());
-      SwiftApiService.getContainer(planName)
+      return dispatch(SwiftApiService.getContainer(planName))
         .then(response => {
-          dispatch(
-            this.receivePlan(
-              planName,
-              normalize(response, arrayOf(planFileSchema)).entities.planFiles
-            )
-          );
+          const planFiles =
+            normalize(response, [planFileSchema]).entities.planFiles || {};
+          dispatch(this.receivePlan(planName, planFiles));
         })
         .catch(error => {
-          logger.error(
-            'Error retrieving plan PlansActions.fetchPlan',
-            error.stack || error
-          );
-          let errorHandler = new SwiftApiErrorHandler(error);
-          errorHandler.errors.forEach(error => {
-            dispatch(NotificationActions.notify(error));
-          });
+          dispatch(handleErrors(error, 'Plan could not be loaded'));
         });
     };
   },
@@ -179,10 +167,13 @@ export default {
     };
   },
 
-  updatePlanFailed(planName) {
+  updatePlanFailed(planName, errors) {
     return {
       type: PlansConstants.UPDATE_PLAN_FAILED,
-      payload: planName
+      payload: {
+        planName,
+        errors
+      }
     };
   },
 
@@ -190,27 +181,24 @@ export default {
     return (dispatch, getState, { getIntl }) => {
       const { formatMessage } = getIntl(getState());
       dispatch(this.updatePlanPending(planName));
-      this._uploadFilesToContainer(planName, fromJS(planFiles), dispatch)
-        .then(() => {
-          dispatch(this.updatePlanSuccess(planName));
-          history.push('/plans/manage');
+      return dispatch(uploadFilesToContainer(planName, planFiles))
+        .then(response =>
           dispatch(
-            NotificationActions.notify({
-              title: formatMessage(messages.planUpdatedNotificationTitle),
-              message: formatMessage(messages.planUpdatedNotificationMessage, {
-                planName: planName
-              }),
-              type: 'success'
+            MistralApiService.runWorkflow(MistralConstants.PLAN_UPDATE, {
+              container: planName
             })
+          )
+        )
+        .catch(error => {
+          dispatch(handleErrors(error, 'Plan update failed', false));
+          dispatch(
+            this.updatePlanFailed(planName, [
+              {
+                title: formatMessage(messages.planUpdateFailed),
+                message: error.message
+              }
+            ])
           );
-          dispatch(this.fetchPlans());
-        })
-        .catch(errors => {
-          logger.error('Error in PlansActions.updatePlan', errors);
-          errors.forEach(error => {
-            dispatch(NotificationActions.notify(error));
-          });
-          dispatch(this.updatePlanFailed(planName));
         });
     };
   },
@@ -219,32 +207,58 @@ export default {
     return (dispatch, getState, { getIntl }) => {
       const { formatMessage } = getIntl(getState());
       dispatch(this.updatePlanPending(planName));
-      SwiftApiService.uploadTarball(planName, file)
+      return dispatch(SwiftApiService.uploadTarball(planName, file))
         .then(response => {
-          dispatch(this.updatePlanSuccess(planName));
-          history.push('/plans/manage');
           dispatch(
-            NotificationActions.notify({
-              title: formatMessage(messages.planUpdatedNotificationTitle),
-              message: formatMessage(messages.planUpdatedNotificationMessage, {
-                planName: planName
-              }),
-              type: 'success'
+            MistralApiService.runWorkflow(MistralConstants.PLAN_UPDATE, {
+              container: planName
             })
           );
-          dispatch(this.fetchPlans());
         })
         .catch(error => {
-          logger.error('Error in PlansActions.updatePlanFromTarball', error);
-          let errorHandler = new SwiftApiErrorHandler(error);
-          errorHandler.errors.forEach(error => {
-            dispatch(NotificationActions.notify(error));
-          });
-          dispatch(this.updatePlanFailed(planName));
+          dispatch(handleErrors(error, 'Plan update failed', false));
+          dispatch(
+            this.updatePlanFailed(planName, [
+              {
+                title: formatMessage(messages.planUpdateFailed),
+                message: error.message
+              }
+            ])
+          );
         });
     };
   },
 
+  updatePlanFinished(payload, history) {
+    return (dispatch, getState, { getIntl }) => {
+      const { formatMessage } = getIntl(getState());
+      const planName = payload.execution.input.container;
+      if (payload.status === 'SUCCESS') {
+        dispatch(this.updatePlanSuccess(planName));
+        dispatch(
+          NotificationActions.notify({
+            title: formatMessage(messages.planUpdatedNotificationTitle),
+            message: formatMessage(messages.planUpdatedNotificationMessage, {
+              planName: planName
+            }),
+            type: 'success'
+          })
+        );
+        dispatch(this.fetchPlans());
+        history.push('/plans/manage');
+      } else {
+        dispatch(
+          this.updatePlanFailed(planName, [
+            {
+              title: formatMessage(messages.planUpdateFailed),
+              message: payload.message
+            }
+          ])
+        );
+      }
+    };
+  },
+
   cancelCreatePlan() {
     return {
       type: PlansConstants.CANCEL_CREATE_PLAN
@@ -270,87 +284,58 @@ export default {
     };
   },
 
-  /*
-   * Uploads a number of files to a container.
-   * Returns a promise which gets resolved when all files are uploaded
-   * or rejected if >= 1 objects fail.
-   * @container: String
-   * @files: Immutable Map
-   */
-  _uploadFilesToContainer(container, files, dispatch) {
-    let uploadedFiles = 0;
-    return when.promise((resolve, reject) => {
-      files.forEach((value, key) => {
-        SwiftApiService.createObject(container, key, value.get('contents'))
-          .then(response => {
-            // On success increase nr of uploaded files.
-            // If this is the last file in the map, resolve the promise.
-            if (uploadedFiles === files.size - 1) {
-              resolve();
-            }
-            uploadedFiles += 1;
-          })
-          .catch(error => {
-            // Reject the promise on the first file that fails.
-            logger.error('Error in PlansActions.createPlan', error);
-            let errorHandler = new SwiftApiErrorHandler(error);
-            reject(errorHandler.errors);
-          });
-      });
-    });
+  createPlan(planName, planFiles) {
+    return dispatch => {
+      dispatch(this.createPlanPending());
+      return dispatch(
+        MistralApiService.runAction(MistralConstants.CREATE_CONTAINER, {
+          container: planName
+        })
+      )
+        .then(response => dispatch(uploadFilesToContainer(planName, planFiles)))
+        .then(response =>
+          dispatch(
+            MistralApiService.runWorkflow(MistralConstants.PLAN_CREATE, {
+              container: planName
+            })
+          )
+        )
+        .catch(error => {
+          dispatch(handleErrors(error, 'Plan creation failed', false));
+          dispatch(
+            this.createPlanFailed([
+              { title: 'Plan creation failed', message: error.message }
+            ])
+          );
+        });
+    };
   },
 
-  createPlan(planName, planFiles) {
-    return (dispatch, getState) => {
+  createPlanFromTarball(planName, file) {
+    return dispatch => {
       dispatch(this.createPlanPending());
-      MistralApiService.runAction(MistralConstants.CREATE_CONTAINER, {
-        container: planName
-      })
-        .then(response => {
-          // Upload all files to container first.
-          this._uploadFilesToContainer(planName, fromJS(planFiles), dispatch)
-            .then(() => {
-              // Once all files are uploaded, start plan creation workflow.
-              MistralApiService.runWorkflow(MistralConstants.PLAN_CREATE, {
-                container: planName
-              })
-                .then(response => {
-                  if (response.state === 'ERROR') {
-                    logger.error('Error in PlansActions.createPlan', response);
-                    dispatch(
-                      NotificationActions.notify({
-                        title: 'Error',
-                        message: response.state_info
-                      })
-                    );
-                    dispatch(this.createPlanFailed());
-                  }
-                })
-                .catch(error => {
-                  logger.error('Error in PlansActions.createPlan', error);
-                  let errorHandler = new MistralApiErrorHandler(error);
-                  errorHandler.errors.forEach(error => {
-                    dispatch(NotificationActions.notify(error));
-                  });
-                  dispatch(this.createPlanFailed());
-                });
-            })
-            .catch(errors => {
-              logger.error('Error in PlansActions.createPlan', errors);
-              // If the file upload fails, just notify the user
-              errors.forEach(error => {
-                dispatch(NotificationActions.notify(error));
-              });
-              dispatch(this.createPlanFailed());
-            });
+      dispatch(
+        MistralApiService.runAction(MistralConstants.CREATE_CONTAINER, {
+          container: planName
         })
+      )
+        .then(response =>
+          dispatch(SwiftApiService.uploadTarball(planName, file))
+        )
+        .then(response =>
+          dispatch(
+            MistralApiService.runWorkflow(MistralConstants.PLAN_CREATE, {
+              container: planName
+            })
+          )
+        )
         .catch(error => {
-          logger.error('Error in PlansActions.createPlan', error);
-          let errorHandler = new MistralApiErrorHandler(error);
-          errorHandler.errors.forEach(error => {
-            dispatch(NotificationActions.notify(error));
-          });
-          dispatch(this.createPlanFailed());
+          dispatch(handleErrors(error, 'Plan creation failed', false));
+          dispatch(
+            this.createPlanFailed([
+              { title: 'Plan creation failed', message: error.message }
+            ])
+          );
         });
     };
   },
@@ -374,67 +359,14 @@ export default {
         history.push('/plans/manage');
       } else {
         dispatch(
-          this.createPlanFailed([{ title: 'Error', message: payload.message }])
+          this.createPlanFailed([
+            { title: 'Plan creation failed', message: payload.message }
+          ])
         );
       }
     };
   },
 
-  createPlanFromTarball(planName, file) {
-    return dispatch => {
-      dispatch(this.createPlanPending());
-
-      MistralApiService.runAction(MistralConstants.CREATE_CONTAINER, {
-        container: planName
-      })
-        .then(response => {
-          SwiftApiService.uploadTarball(planName, file)
-            .then(response => {
-              MistralApiService.runWorkflow(MistralConstants.PLAN_CREATE, {
-                container: planName
-              })
-                .then(response => {
-                  if (response.state === 'ERROR') {
-                    logger.error(
-                      'Error in PlansActions.createPlanFromTarball',
-                      response
-                    );
-                    dispatch(
-                      this.createPlanFailed([
-                        { title: 'Error', message: response.state_info }
-                      ])
-                    );
-                  }
-                })
-                .catch(error => {
-                  logger.error(
-                    'Error in workflow in PlansActions.createPlanFromTarball',
-                    error
-                  );
-                  let errorHandler = new MistralApiErrorHandler(error);
-                  dispatch(this.createPlanFailed(errorHandler.errors));
-                });
-            })
-            .catch(error => {
-              logger.error(
-                'Error in Swift in PlansActions.createPlanFromTarball',
-                error
-              );
-              let errorHandler = new SwiftApiErrorHandler(error);
-              dispatch(this.createPlanFailed(errorHandler.errors));
-            });
-        })
-        .catch(error => {
-          logger.error('Error in PlansActions.createPlan', error);
-          let errorHandler = new MistralApiErrorHandler(error);
-          errorHandler.errors.forEach(error => {
-            dispatch(NotificationActions.notify(error));
-          });
-          dispatch(this.createPlanFailed());
-        });
-    };
-  },
-
   deletePlanPending(planName) {
     return {
       type: PlansConstants.DELETE_PLAN_PENDING,
@@ -461,9 +393,11 @@ export default {
       const { formatMessage } = getIntl(getState());
       dispatch(this.deletePlanPending(planName));
       history.push('/plans/manage');
-      MistralApiService.runAction(MistralConstants.PLAN_DELETE, {
-        container: planName
-      })
+      return dispatch(
+        MistralApiService.runAction(MistralConstants.PLAN_DELETE, {
+          container: planName
+        })
+      )
         .then(response => {
           dispatch(this.deletePlanSuccess(planName));
           dispatch(
@@ -477,15 +411,10 @@ export default {
           );
         })
         .catch(error => {
-          logger.error(
-            'Error deleting plan MistralApiService.runAction',
-            error
+          dispatch(
+            handleErrors(error, `Plan ${planName} could not be deleted`)
           );
           dispatch(this.deletePlanFailed(planName));
-          let errorHandler = new MistralApiErrorHandler(error);
-          errorHandler.errors.forEach(error => {
-            dispatch(NotificationActions.notify(error));
-          });
         });
     };
   },
@@ -511,6 +440,27 @@ export default {
     };
   },
 
+  deployPlan(planName) {
+    return dispatch => {
+      dispatch(this.deployPlanPending(planName));
+      dispatch(
+        MistralApiService.runWorkflow(MistralConstants.DEPLOYMENT_DEPLOY_PLAN, {
+          container: planName,
+          timeout: 240
+        })
+      )
+        .then(response => {
+          dispatch(StackActions.fetchStacks());
+        })
+        .catch(error => {
+          dispatch(
+            handleErrors(error, `Plan ${planName} could not be deployed`)
+          );
+          dispatch(this.deployPlanFailed(planName));
+        });
+    };
+  },
+
   deployPlanFinished(payload) {
     return (dispatch, getState, { getIntl }) => {
       const { formatMessage } = getIntl(getState());
@@ -530,26 +480,6 @@ export default {
     };
   },
 
-  deployPlan(planName) {
-    return dispatch => {
-      dispatch(this.deployPlanPending(planName));
-      MistralApiService.runWorkflow(MistralConstants.DEPLOYMENT_DEPLOY_PLAN, {
-        container: planName,
-        timeout: 240
-      })
-        .then(response => {
-          dispatch(StackActions.fetchStacks());
-        })
-        .catch(error => {
-          dispatch(this.deployPlanFailed(planName));
-          let errorHandler = new MistralApiErrorHandler(error);
-          errorHandler.errors.forEach(error => {
-            dispatch(NotificationActions.notify(error));
-          });
-        });
-    };
-  },
-
   exportPlanPending(planName) {
     return {
       type: PlansConstants.EXPORT_PLAN_PENDING,
@@ -571,6 +501,20 @@ export default {
     };
   },
 
+  exportPlan(planName) {
+    return dispatch => {
+      dispatch(this.exportPlanPending(planName));
+      dispatch(
+        MistralApiService.runWorkflow(MistralConstants.PLAN_EXPORT, {
+          plan: planName
+        })
+      ).catch(error => {
+        dispatch(handleErrors(error, `Plan ${planName} could not be exported`));
+        dispatch(this.exportPlanFailed(planName));
+      });
+    };
+  },
+
   exportPlanFinished(payload) {
     return (dispatch, getState, { getIntl }) => {
       const { formatMessage } = getIntl(getState());
@@ -587,35 +531,32 @@ export default {
         let urlParser = document.createElement('a');
         urlParser.href = payload.tempurl;
         let url = urlParser.hostname;
-        urlParser.href = getAppConfig().swift;
+        urlParser.href = getServiceUrl(getState(), 'swift');
         let swiftUrl = urlParser.hostname;
         dispatch(
           this.exportPlanSuccess(payload.tempurl.replace(url, swiftUrl))
         );
       }
     };
-  },
-
-  exportPlan(planName) {
-    return dispatch => {
-      dispatch(this.exportPlanPending(planName));
-      MistralApiService.runWorkflow(MistralConstants.PLAN_EXPORT, {
-        plan: planName
-      })
-        .then(response => {
-          if (response.state === 'ERROR') {
-            logger.error('Error in PlansActions.exportPlan', response);
-            dispatch(this.exportPlanFailed(planName));
-          }
-        })
-        .catch(error => {
-          logger.error('Error in PlansActions.exportPlan', error);
-          dispatch(this.exportPlanFailed(planName));
-          let errorHandler = new MistralApiErrorHandler(error);
-          errorHandler.errors.forEach(error => {
-            dispatch(NotificationActions.notify(error));
-          });
-        });
-    };
   }
 };
+
+/*
+  * Uploads a number of files to a container.
+  * Returns a promise which gets resolved when all files are uploaded
+  * or rejected if >= 1 objects fail.
+  * @container: String
+  * @files: Object
+  */
+export const uploadFilesToContainer = (container, files) => dispatch =>
+  when.all(
+    Object.keys(files).map(fileName =>
+      dispatch(
+        SwiftApiService.createObject(
+          container,
+          fileName,
+          files[fileName].contents
+        )
+      )
+    )
+  );
